#!/usr/local/bin/perl -T
#
#	archie.cgi --- CGI script for Archiesrv
#
#	Copyright (C) 1996,1997  Hiroo Hayashi
#	
#	Author: Hiroo Hayashi <hiroo.hayashi@toshiba.co.jp>
#	$Id: archie.cgi,v 2.11 1997-07-09 16:08:15+09 hayashi Exp $
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#	This program requires CGI module.  CGI module is included in
#	libwww module.  You can get it from CPAN.  The author
#	currently uses "libwww-perl-5.00.tar.gz".

use strict;
use CGI;
use vars qw($DEBUG $ROOTDIR $ARCHIE @SITE_ATTR $ADM $ADM_ADRS);

$DEBUG = 0;
$ROOTDIR = '/usr/local/archiesrv';
$ARCHIE = "$ROOTDIR/bin/archie";
$ENV{PATH} = '/usr/bin:/bin:/ucb';

unshift (@INC, $ROOTDIR);
require 'archiesrv.conf';

################################################################
my $query = new CGI;
my $referer = $query->referer;
my $script_name = $query->script_name;

if ($referer =~ m|$script_name/?$|) {
    show_result($query);
} else {
    show_menu($query);
}
exit 0;

################################################################
sub show_result {
    my $q = $_[0];

    print $q->header;
    print $q->start_html(-title=>'Archie Result',
			 -author=>'hiroo.hayashi@toshiba.co.jp');

    # show debugging info.
    if ($DEBUG) {
	print $q->dump;
	show_env();
	show_session_var($q);
    }

    print "<H3>Results of Archie Search</H3>\n";
    print "<hr>\n"; 

    # make command string for archie
    my %type = ( e   => '-e',
		 S   => '-s',    C => '-c',	R => '-r',
		 's' => '-e -s', c => '-e -c',	r => '-e -r');
    
    my %nice = ('Not Nice At All' => 0,		'Nice'		=> 500,
		'Nicer'		  => 1000,	'Very Nice'	=> 5000,
		'Extremely Nice'  => 10000,	'Nicest'	=> 32765);
    
    $_ = $q->param('hits');
    my $hits = /^(\d+)$/ ? $1 : '';

    $_ = $q->param('pattern');
    my $query = /^([\w +?.*()\[\]{}|\\^\-\$]+)$/ ? $1 : '';

    $_ = $q->param('filterpattern');
    my $filter = /^([\w +?.*()\[\]{}|\\^\-\$]+)$/ ? $1 : '';

    my $domain = join(':', $q->param('domain'));

    my @cmd = ($ARCHIE,
	       '-l',
	       $type{$q->param('type')},
	       '-N', $nice{$q->param('nice')});
    push(@cmd, '-t') if $q->param('order') eq 'date';
    push(@cmd, '-P', $filter) if $q->param('filter') eq 'pos';
    push(@cmd, '-P', "!$filter") if $q->param('filter') eq 'neg';
    push(@cmd, '-D', $domain) if $domain;
    push(@cmd, '-m', $hits) if $hits;
    push(@cmd, $query);
		   
    ################################################################
    # main loop
    print "@cmd<br><br>\n" if $DEBUG;
    # cf. perlsec(1)
    my $pid = open (ARCHIE, "-|");
    die "cannot fork:$!\n" unless defined $pid;
    unless ($pid) {		# child
	$> = $<; $) = $(;	# change to real uid/gid for sanity
	exec @cmd;
	die "cannot exec $ARCHIE : $!\n";
    }
    my $n = 0;
    my $lasthost = my $lastdir = '';
    while (<ARCHIE>) {
	my @info = split(' ');
	my ($host, $mode, $link, $size,
	    $month, $day, $year_hour, $file, $dir) = @info;
	
	print "<menu>\n" unless $n;
	if ($host ne $lasthost) {
	    print "</pre>" if $n;
	    print "<li>  $host<pre>";
	}
	print "$dir\n" if $dir ne $lastdir;
	printf "\t%7d %3s %2d %-5s\t", $size, $month, $day, $year_hour;
	(my $path = "$host/$dir/$file") =~ s|/+|/|g;
	print "<a href=\"ftp://$path\">$file</a>\n";
	
	$n++; $lasthost = $host; $lastdir = $dir;
    }
    close(ARCHIE);
    
    ################################################################
    # epilogue
    if ($n) {
	print "</menu>\n";
	print "<hr>Total number of hits are $n<br>\n";
    } else {
	print "No hit for \"$query\"<br>\n";
    }

    print $q->end_html, "\n";
}

################################################################
sub show_menu {
    my $q = $_[0];

    print $q->header;
    print $q->start_html(-title=>'Archie Service',
			 -author=>'hiroo.hayashi@toshiba.co.jp');

    # show debugging info.
    if ($DEBUG) {
	print $q->dump;
	show_env();
	show_session_var($q);
    }

    print("<H3>Archie Service</H3>\n<HR>\n",
	  $q->startform(-method=>'POST',
			-action=>$ENV{SCRIPT_NAME}));

    print('Search for:',
	  $q->textfield(-name=>'pattern'),
	  $q->submit(-value=>'Submit'),
	  $q->reset,
	  "<P>\n");

    print "Anonymous FTP Servers to Search:\n<blockquote>\n";

    my $site_ref;
    foreach $site_ref (@SITE_ATTR) {
	my $site = $site_ref->{address};
	print('<INPUT TYPE="checkbox" NAME="domain" ',
	      "VALUE=\"$site\"");
	print ' CHECKED' if $site_ref->{checked};
	print ">$site<br>\n";
    }

    print "</blockquote><p>\n\n";
    print("Search method:",
	  $q->popup_menu(-name=>'type',
			 -values=>['s','S','C','R','s','c','r'],
			 -default=>'S',
			 -labels=>
			 {'e'=>'Exact Match',
			  'S'=>'Case Insensitive Substring Match',
			  'C'=>'Case Sensitive Substring Match',
			  'R'=>'Regular Expression Match',
			  's'=>'Exact Match else Case Insensitive Substring Match',
			  'c'=>'Exact Match else Case Sensitive Substring Match',
			  'r'=>'Exact Match else Regular Expression Match'}),
	  "<br>\n");

    print("Sort:",
	  $q->popup_menu(-name=>'order',
			 -values=>['host','date'],
			 -default=>'host',
			 -labels=>{'host'=>'by Host',
				   'date'=>'by Date'}),
	  "<br>\n");

    print("PATH Filter:",
	  $q->popup_menu(-name=>'filter',
			 -values=>['off','neg', 'pos'],
			 -default=>'off',
			 -labels=>{'off'=>'Off',
				   'neg'=>'Search files whose paths don\'t match with',
				   'pos'=>'Search files whose paths match with'}),
	  $q->textfield(-name=>'filterpattern',
			-default=>'FreeBSD|Linux|linux|NetBSD',
			-size=>'32'),
	  "<br>\n");

    print("Maximum numbers to search (&lt; 100):",
	  $q->textfield(-name=>'hits',
			-default=>'95',
			-size=>'3'),
	  "<br>\n");
    
    print("Nice value (process priority):",
	  $q->popup_menu(-name=>'nice',
			 -default=>'Nice',
			 -values=>['Not Nice At All',
				   'Nice',
				   'Nicer',
				   'Very Nice',
				   'Extremely Nice',
				   'Nicest']));

    print $q->end_form;

    print "<hr>\n";
    print "<address>$ADM &lt;$ADM_ADRS&gt;</address>\n";

    print $q->end_html, "\n";
}

################################################################
# print environment variables for debugging
sub show_env {
    print "<hr>\n<pre>\n";
    chop(my $pwd = `pwd`);
    print "pwd=$pwd\n";
    print "\$0=$0\n";
    print "\@ARGV=@ARGV\n";
    print "QUERY_STRING=$ENV{QUERY_STRING}\n";
    print "PATH_INFO=$ENV{PATH_INFO}\n";
    print "PATH_TRANSLATED=$ENV{PATH_TRANSLATED}\n";
    print "REQUEST_METHOD=$ENV{REQUEST_METHOD}\n";
    print "CONTENT_LENGTH=$ENV{CONTENT_LENGTH}\n";
    print "SERVER_NAME=$ENV{SERVER_NAME}\n";
    print "SCRIPT_NAME=$ENV{SCRIPT_NAME}\n";
    print "GATEWAY_INTERFACE=$ENV{GATEWAY_INTERFACE}\n";
    print "</pre>\n<hr>\n";
}

sub show_session_var {
    my $q = $_[0];

    print "<hr>\n<pre>\n";

    my @accept = $q->accept;
    print "accept()=@accept\n";
    print("auth_type()=",	$q->auth_type, "\n",
	  "raw_cookie()=",	$q->raw_cookie, "\n",
	  "path_info()=",	$q->path_info, "\n",
	  "path_translated()=",	$q->path_translated, "\n",
	  "query_string()=",	$q->query_string, "\n",
	  "referer()=",		$q->referer, "\n",
	  "remote_addr()=",	$q->remote_addr, "\n",
	  "remote_ident()=",	$q->remote_ident, "\n",
	  "remote_host()=",	$q->remote_host, "\n",
	  "remote_user()=",	$q->remote_user, "\n",
	  "request_method()=",	$q->request_method, "\n",
	  "script_name()=",	$q->script_name, "\n",
	  "server_name()=",	$q->server_name, "\n",
	  "server_port()=",	$q->server_port, "\n",
	  "user_agent()=",	$q->user_agent, "\n",
	  "user_name()=",	$q->user_name, "\n");

    print "</pre>\n<hr>\n";
}
